@@ -9,7 +9,6 @@ import copy
 import io
 import logging
 import os
-import re
 import shutil
 
 import pylru
@@ -21,12 +20,11 @@ except ImportError:
 
 from contextlib import contextmanager
 
-from emoji import demojize
 from past.builtins import basestring
 
 import octoprint.filemanager
 from octoprint.util import atomic_write, is_hidden_path, time_this, to_bytes, to_unicode
-from octoprint.vendor.awesome_slugify import Slugify
+from octoprint.util.files import sanitize_filename
 
 
 class StorageInterface(object):
@@ -34,6 +32,7 @@ class StorageInterface(object):
     Interface of storage adapters for OctoPrint.
     """
 
+    # noinspection PyUnreachableCode
     @property
     def analysis_backlog(self):
         """
@@ -48,6 +47,7 @@ class StorageInterface(object):
         return
         yield
 
+    # noinspection PyUnreachableCode
     def analysis_backlog_for_path(self, path=None):
         # empty generator pattern, yield is intentionally unreachable
         return
@@ -468,29 +468,14 @@ class LocalFileStorage(StorageInterface):
     This storage type implements :func:`path_on_disk`.
     """
 
-    _UNICODE_VARIATIONS = re.compile("[\uFE00-\uFE0F]", re.U)
-
-    @classmethod
-    def _no_unicode_variations(cls, text):
-        return cls._UNICODE_VARIATIONS.sub("", text)
-
-    _SLUGIFY = Slugify()
-    _SLUGIFY.safe_chars = "-_.()[] "
-
-    @classmethod
-    def _slugify(cls, text):
-        text = to_unicode(text)
-        text = cls._no_unicode_variations(text)
-        text = demojize(text, delimiters=("", ""))
-        return cls._SLUGIFY(text)
-
-    def __init__(self, basefolder, create=False):
+    def __init__(self, basefolder, create=False, really_universal=False):
         """
         Initializes a ``LocalFileStorage`` instance under the given ``basefolder``, creating the necessary folder
         if necessary and ``create`` is set to ``True``.
 
-        :param string basefolder: the path to the folder under which to create the storage
-        :param bool create:       ``True`` if the folder should be created if it doesn't exist yet, ``False`` otherwise
+        :param string basefolder:     the path to the folder under which to create the storage
+        :param bool create:           ``True`` if the folder should be created if it doesn't exist yet, ``False`` otherwise
+        :param bool really_universal: ``True`` if the file names should be forced to really universal, ``False`` otherwise
         """
         self._logger = logging.getLogger(__name__)
 
@@ -503,6 +488,8 @@ class LocalFileStorage(StorageInterface):
                 code=StorageError.INVALID_DIRECTORY,
             )
 
+        self._really_universal = really_universal
+
         import threading
 
         self._metadata_lock_mutex = threading.RLock()
@@ -1181,22 +1168,10 @@ class LocalFileStorage(StorageInterface):
     def sanitize_name(self, name):
         """
         Raises a :class:`ValueError` for a ``name`` containing ``/`` or ``\\``. Otherwise
-        slugifies the given ``name`` by converting it to ASCII, leaving ``-``, ``_``, ``.``,
-        ``(``, and ``)`` as is.
+        sanitizes the given ``name`` using ``octoprint.files.sanitize_filename``. Also
+        strips any leading ``.``.
         """
-        name = to_unicode(name)
-
-        if name is None:
-            return None
-
-        if "/" in name or "\\" in name:
-            raise ValueError("name must not contain / or \\")
-
-        result = self._slugify(name).replace(" ", "_")
-        if result and result != "." and result != ".." and result[0] == ".":
-            # hidden files under *nix
-            result = result[1:]
-        return result
+        return sanitize_filename(name, really_universal=self._really_universal)
 
     def sanitize_path(self, path):
         """
@@ -1215,7 +1190,10 @@ class LocalFileStorage(StorageInterface):
         path_elements = path.split("/")
         joined_path = self.basefolder
         for path_element in path_elements:
-            joined_path = os.path.join(joined_path, self.sanitize_name(path_element))
+            if path_element == ".." or path_element == ".":
+                joined_path = os.path.join(joined_path, path_element)
+            else:
+                joined_path = os.path.join(joined_path, self.sanitize_name(path_element))
         path = os.path.realpath(joined_path)
         if not path.startswith(self.basefolder):
             raise ValueError(
@@ -1376,7 +1354,7 @@ class LocalFileStorage(StorageInterface):
                 continue
             statistics["averagePrintTime"][printer_profile] = sum(
                 former_print_times[printer_profile]
-            ) / float(len(former_print_times[printer_profile]))
+            ) / len(former_print_times[printer_profile])
 
         for printer_profile in last_print:
             if not last_print[printer_profile]:
@@ -1524,6 +1502,7 @@ class LocalFileStorage(StorageInterface):
         logtarget=__name__ + ".timings",
         message="{func}({func_args},{func_kwargs}) took {timing:.2f}ms",
         incl_func_args=True,
+        log_enter=True,
     )
     def _list_folder(self, path, base="", force_refresh=False, **kwargs):
         def get_size(nodes):
@@ -1551,11 +1530,8 @@ class LocalFileStorage(StorageInterface):
         try:
             with self._filelist_cache_mutex:
                 cache = self._filelist_cache.get(path)
-                if (
-                    not force_refresh
-                    and cache
-                    and cache[0] >= self.last_modified(path, recursive=True)
-                ):
+                lm = self.last_modified(path, recursive=True)
+                if not force_refresh and cache and cache[0] >= lm:
                     return enrich_folders(cache[1])
 
                 metadata = self._get_metadata(path)
@@ -1696,7 +1672,7 @@ class LocalFileStorage(StorageInterface):
                         continue
 
                 self._filelist_cache[path] = (
-                    self.last_modified(path, recursive=True),
+                    lm,
                     result,
                 )
                 return enrich_folders(result)
@@ -1891,7 +1867,7 @@ class LocalFileStorage(StorageInterface):
             try:
                 with atomic_write(metadata_path, mode="wb") as f:
                     f.write(
-                        to_bytes(json.dumps(metadata, indent=4, separators=(",", ": ")))
+                        to_bytes(json.dumps(metadata, indent=2, separators=(",", ": ")))
                     )
             except Exception:
                 self._logger.exception(
@@ -1965,7 +1941,7 @@ class LocalFileStorage(StorageInterface):
                 return
 
             with atomic_write(metadata_path_json, mode="wb") as f:
-                f.write(to_bytes(json.dumps(metadata, indent=4, separators=(",", ": "))))
+                f.write(to_bytes(json.dumps(metadata, indent=2, separators=(",", ": "))))
 
             try:
                 os.remove(metadata_path_yaml)
