@@ -31,7 +31,7 @@ $(function () {
             5
         );
 
-        self.unread = ko.observable();
+        self.unread = ko.observable(false);
         self.hiddenChannels = [];
         self.channelNotifications = {};
 
@@ -209,7 +209,6 @@ $(function () {
                         }
                     })
                     .css({
-                        "width": "auto",
                         "margin-left": function () {
                             return -($(this).width() / 2);
                         }
