@@ -188,6 +188,7 @@ class VirtualPrinter(object):
 
         self._temperature_reporter = None
         self._sdstatus_reporter = None
+        self._pos_reporter = None
 
         self.current_line = 0
         self.lastN = 0
@@ -202,7 +203,7 @@ class VirtualPrinter(object):
 
         self._received_lines = 0
         self._resend_every_n = 0
-        self._calculate_resend_every_n(self._settings.get(["resend_ratio"]))
+        self._calculate_resend_every_n(self._settings.get_int(["resend_ratio"]))
 
         self._dont_answer = False
 
@@ -607,7 +608,7 @@ class VirtualPrinter(object):
     def _gcode_M20(self, data):
         # type: (str) -> None
         if self._sdCardReady:
-            self._listSd()
+            self._listSd(incl_long="L" in data)
 
     # noinspection PyUnusedLocal
     def _gcode_M21(self, data):
@@ -704,26 +705,7 @@ class VirtualPrinter(object):
     # noinspection PyUnusedLocal
     def _gcode_M114(self, data):
         # type: (str) -> bool
-        m114FormatString = self._settings.get(["m114FormatString"])
-        e = dict((index, value) for index, value in enumerate(self._lastE))
-        e["current"] = self._lastE[self.currentExtruder]
-        e["all"] = " ".join(
-            [
-                "E{}:{}".format(num, self._lastE[self.currentExtruder])
-                for num in range(self.extruderCount)
-            ]
-        )
-        output = m114FormatString.format(
-            x=self._lastX,
-            y=self._lastY,
-            z=self._lastZ,
-            e=e,
-            f=self._lastF,
-            a=int(self._lastX * 100),
-            b=int(self._lastY * 100),
-            c=int(self._lastZ * 100),
-        )
-
+        output = self._generatePositionOutput()
         if not self._okBeforeCommandOutput:
             ok = self._ok()
             if ok:
@@ -745,7 +727,26 @@ class VirtualPrinter(object):
         # type: (str) -> None
         # we'll just use this to echo a message, to allow playing around with pause triggers
         if self._echoOnM117:
-            self._send("echo:%s" % re.search(r"M117\s+(.*)", data).group(1))
+            try:
+                self._send("echo:%s" % re.search(r"M117\s+(.*)", data).group(1))
+            except AttributeError:
+                self._send("echo:")
+
+    def _gcode_M154(self, data):
+        # type: (str) -> None
+        matchS = re.search(r"S([0-9]+)", data)
+        if matchS is not None:
+            interval = int(matchS.group(1))
+            if self._pos_reporter is not None:
+                self._pos_reporter.cancel()
+
+            if interval > 0:
+                self._pos_reporter = RepeatedTimer(
+                    interval, lambda: self._send(self._generatePositionOutput())
+                )
+                self._pos_reporter.start()
+            else:
+                self._pos_reporter = None
 
     def _gcode_M155(self, data):
         # type: (str) -> None
@@ -809,7 +810,7 @@ class VirtualPrinter(object):
     # noinspection PyUnusedLocal
     def _gcode_G20(self, data):
         # type: (str) -> None
-        self._unitModifier = 1.0 / 2.54
+        self._unitModifier = 1 / 2.54
         if self._lastX is not None:
             self._lastX *= 2.54
         if self._lastY is not None:
@@ -866,7 +867,7 @@ class VirtualPrinter(object):
 
         _timeout = 0
         if matchP:
-            _timeout = float(matchP.group(1)) / 1000.0
+            _timeout = float(matchP.group(1)) / 1000
         elif matchS:
             _timeout = float(matchS.group(1))
 
@@ -1091,7 +1092,7 @@ class VirtualPrinter(object):
                 if key == "S":
                     pass
                 else:
-                    self._virtual_eeprom.eeprom["advanced"]["params"][material_no][
+                    self._virtual_eeprom.eeprom["material"]["params"][material_no][
                         key
                     ] = float(value)
 
@@ -1137,7 +1138,7 @@ class VirtualPrinter(object):
         config = self._virtual_eeprom.eeprom["material"]
         lines = []
         for material in ["0", "1"]:
-            line = "echo: " + config["command"] + "S" + material
+            line = "echo: " + config["command"] + " S" + material
             for param, saved_value in config["params"][material].items():
                 line = line + " " + param + str(saved_value)
             lines.append(line)
@@ -1412,10 +1413,13 @@ class VirtualPrinter(object):
             except Exception:
                 self._logger.exception("While handling %r", data)
 
-    def _listSd(self):
+    def _listSd(self, incl_long=False):
         if self._settings.get_boolean(["sdFiles", "size"]):
-            if self._settings.get_boolean(["sdFiles", "longname"]):
-                line = "{dosname} {size} {name}"
+            if self._settings.get_boolean(["sdFiles", "longname"]) or incl_long:
+                if self._settings.get_boolean(["sdFiles", "longname_quoted"]):
+                    line = '{dosname} {size} "{name}"'
+                else:
+                    line = "{dosname} {size} {name}"
             else:
                 line = "{dosname} {size}"
         else:
@@ -1495,6 +1499,29 @@ class VirtualPrinter(object):
         else:
             self._send("Not SD printing")
 
+    def _generatePositionOutput(self):
+        # type: () -> str
+        m114FormatString = self._settings.get(["m114FormatString"])
+        e = {index: value for index, value in enumerate(self._lastE)}
+        e["current"] = self._lastE[self.currentExtruder]
+        e["all"] = " ".join(
+            [
+                "E{}:{}".format(num, self._lastE[self.currentExtruder])
+                for num in range(self.extruderCount)
+            ]
+        )
+        output = m114FormatString.format(
+            x=self._lastX,
+            y=self._lastY,
+            z=self._lastZ,
+            e=e,
+            f=self._lastF,
+            a=int(self._lastX * 100),
+            b=int(self._lastY * 100),
+            c=int(self._lastZ * 100),
+        )
+        return output
+
     def _generateTemperatureOutput(self):
         # type: () -> str
         if self._settings.get_boolean(["repetierStyleTargetTemperature"]):
@@ -1639,8 +1666,8 @@ class VirtualPrinter(object):
             except ValueError:
                 pass
 
-        speedXYZ = self._lastF * (self._feedrate_multiplier / 100.0)
-        speedE = self._lastF * (self._flowrate_multiplier / 100.0)
+        speedXYZ = self._lastF * (self._feedrate_multiplier / 100)
+        speedE = self._lastF * (self._flowrate_multiplier / 100)
         if speedXYZ == 0:
             speedXYZ = 999999999999
         if speedE == 0:
@@ -1653,10 +1680,10 @@ class VirtualPrinter(object):
                 pass
             else:
                 if self._relative or self._lastX is None:
-                    duration = max(duration, x * self._unitModifier / speedXYZ * 60.0)
+                    duration = max(duration, x * self._unitModifier / speedXYZ * 60)
                 else:
                     duration = max(
-                        duration, (x - self._lastX) * self._unitModifier / speedXYZ * 60.0
+                        duration, (x - self._lastX) * self._unitModifier / speedXYZ * 60
                     )
 
                 if self._relative and self._lastX is not None:
@@ -1670,10 +1697,10 @@ class VirtualPrinter(object):
                 pass
             else:
                 if self._relative or self._lastY is None:
-                    duration = max(duration, y * self._unitModifier / speedXYZ * 60.0)
+                    duration = max(duration, y * self._unitModifier / speedXYZ * 60)
                 else:
                     duration = max(
-                        duration, (y - self._lastY) * self._unitModifier / speedXYZ * 60.0
+                        duration, (y - self._lastY) * self._unitModifier / speedXYZ * 60
                     )
 
                 if self._relative and self._lastY is not None:
@@ -1687,10 +1714,10 @@ class VirtualPrinter(object):
                 pass
             else:
                 if self._relative or self._lastZ is None:
-                    duration = max(duration, z * self._unitModifier / speedXYZ * 60.0)
+                    duration = max(duration, z * self._unitModifier / speedXYZ * 60)
                 else:
                     duration = max(
-                        duration, (z - self._lastZ) * self._unitModifier / speedXYZ * 60.0
+                        duration, (z - self._lastZ) * self._unitModifier / speedXYZ * 60
                     )
 
                 if self._relative and self._lastZ is not None:
@@ -1705,10 +1732,10 @@ class VirtualPrinter(object):
             else:
                 lastE = self._lastE[self.currentExtruder]
                 if self._relative or lastE is None:
-                    duration = max(duration, e * self._unitModifier / speedE * 60.0)
+                    duration = max(duration, e * self._unitModifier / speedE * 60)
                 else:
                     duration = max(
-                        duration, (e - lastE) * self._unitModifier / speedE * 60.0
+                        duration, (e - lastE) * self._unitModifier / speedE * 60
                     )
 
                 if self._relative and lastE is not None:
@@ -2295,7 +2322,7 @@ class CharCountingQueue(queue.Queue):
                 endtime = monotonic_time() + timeout
                 while not self._will_it_fit(item):
                     remaining = endtime - monotonic_time()
-                    if remaining <= 0.0:
+                    if remaining <= 0:
                         raise queue.Full
                     self.not_full.wait(remaining)
 
